/***************************************************************************
 $RCSfile$
                             -------------------
    cvs         : $Id$
    begin       : Mon Mar 01 2004
    copyright   : (C) 2004 by Martin Preuss
    email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "kbanking_p.h"
#include "ktimport.h"
#include "flagstaff.h"
#include "checkduplicates.h"
#include "transactionwindow.h"
#include "selecttransfers.h"
#include "edittransaction.h"
#include "editeutransaction.h"
#include "editsto.h"
#include "mainwindow.h"
#include "cfgmoduleqbm.h"

#include "report.h"
#include "simplereport.h"
#include "summaryreport.h"
#include "categoryreport.h"
#include "graphreport.h"
/* Note: add new reports to fn _sampleReportModules() */

#include <assert.h>
#include <qstring.h>
#include <qmessagebox.h>
#include <qinputdialog.h>
#include <qapplication.h>
#include <qdatetime.h>

#include <aqbanking/jobsingletransfer.h>
#include <aqbanking/jobsingledebitnote.h>
#include <aqbanking/jobeutransfer.h>
#include <aqbanking/jobcreatesto.h>
#include <aqbanking/jobcreatedatedtransfer.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/text.h>

#include <qwidget.h>


KBanking::KBanking(const char *appname,
                   const char *fname,
                   const char *name)
:App(appname, fname)
,_mainWindow(0)
,_lastWidgetId(0)
,_logLevel(GWEN_LoggerLevel_Info)
,_jobQueue(NULL)
,_useColoursForCategoryText(true)
,_lastReportId(0){
  _flagStaff=new FlagStaff();
  _cfgModule=new CfgModuleQbm(this);
  _jobQueue=AB_Job_List2_new();
  setAppConfigModule(_cfgModule);
}



KBanking::~KBanking(){
  AB_Job_List2_FreeAll(_jobQueue);
  delete _cfgModule;
  delete _flagStaff;
}



FlagStaff *KBanking::flagStaff(){
  return _flagStaff;
}



std::string KBanking::tr(const char *s) {
  return QBanking::QStringToUtf8String(QWidget::tr(s));
}



int KBanking::executeQueue(AB_IMEXPORTER_CONTEXT *ctx){
  int rv;
  AB_JOB_LIST2_ITERATOR *jit;

  rv=QBanking::executeJobs(_jobQueue, ctx, 0);
  jit=AB_Job_List2_First(_jobQueue);
  if (jit) {
    AB_JOB *j;

    j=AB_Job_List2Iterator_Data(jit);
    while(j) {
      const AB_TRANSACTION *t=NULL;

      switch(AB_Job_GetType(j)) {
      case AB_Job_TypeTransfer:
	t=AB_JobSingleTransfer_GetTransaction(j);
	break;
      case AB_Job_TypeDebitNote:
	t=AB_JobSingleDebitNote_GetTransaction(j);
	break;
      case AB_Job_TypeEuTransfer:
	t=AB_JobEuTransfer_GetTransaction(j);
	break;
      default:
        break;
      }

      if (t) {
	RefPointer<Transfer> rt;

	rt=findTransferByJobId(AB_Job_GetJobId(j));
	if (rt.isValid()) {
	  switch(AB_Job_GetStatus(j)) {
	  case AB_Job_StatusPending:
	    rt.ref().setStatus(Transfer::StatusPending);
	    break;
	  case AB_Job_StatusFinished:
	    rt.ref().setStatus(Transfer::StatusOk);
	    break;
	  case AB_Job_StatusError:
	    rt.ref().setStatus(Transfer::StatusError);
	    break;
	  default:
	    rt.ref().setStatus(Transfer::StatusUnknown);
	    break;
	  }
	}
	else {
	  DBG_ERROR(0, "Transaction %08x not found",
		    AB_Job_GetJobId(j));
	}
      }
      else {
        DBG_ERROR(0, "No transaction in this job, ignoring");
      }

      j=AB_Job_List2Iterator_Next(jit);
    }
    AB_Job_List2Iterator_free(jit);
  }

  AB_Job_List2_FreeAll(_jobQueue);
  _jobQueue=AB_Job_List2_new();
  _flagStaff->queueUpdated();
  transfersUpdated();

  return rv;
}



void KBanking::transfersUpdated(){
  _flagStaff->transfersUpdated();
}



void KBanking::standingOrdersUpdated(){
  _flagStaff->standingOrdersUpdated();
}



bool KBanking::importContext(AB_IMEXPORTER_CONTEXT *ctx,
                             uint32_t flags){
  KTransactionImporter imp(this, ctx, flags);

  if (flags & QBANKING_IMPORTER_FLAGS_AS_ORDERS) {
    AB_IMEXPORTER_ACCOUNTINFO *ai;
    std::list<RefPointer<Transfer> > tl;
    std::list<RefPointer<Transfer> > stl;

    ai=AB_ImExporterContext_GetFirstAccountInfo(ctx);
    if (!ai) {
      DBG_ERROR(0, "No accounts");
    }
    while(ai) {
      const AB_TRANSACTION *t;

      DBG_DEBUG(0, "Checking account (%s/%s)",
                AB_ImExporterAccountInfo_GetBankCode(ai),
                AB_ImExporterAccountInfo_GetAccountNumber(ai));
      t=AB_ImExporterAccountInfo_GetFirstTransaction(ai);
      while(t) {
        Transfer *tr;

        if (AB_Transaction_GetType(t)==AB_Transaction_TypeDebitNote)
          tr=new Transfer(t, Transfer::TransferType_DebitNote);
        else
          tr=new Transfer(t, Transfer::TransferType_Simple);
        tl.push_back(tr);
        t=AB_ImExporterAccountInfo_GetNextTransaction(ai);
      } /* while */

      ai=AB_ImExporterContext_GetNextAccountInfo(ctx);
    } /* while accounts */

    /* now we have a list of transfer orders */
    if (!SelectTransfers::selectTransfers(this, tl, stl)) {
      return false;
    }
    else {
      std::list<RefPointer<Transfer> >::iterator it;
      int i=0;
      int rv = GWEN_ERROR_USER_ABORTED;

      for (it=stl.begin(); it!=stl.end(); it++) {
        switch((*it).ref().getTransferType()) {
        case Transfer::TransferType_Simple:
          rv=transferFromTemplate((*it).cast<Transaction>(),
                                  getParentWidget());
          break;
        case Transfer::TransferType_DebitNote:
          rv=debitNoteFromTemplate((*it).cast<Transaction>(),
                                   getParentWidget());
          break;
        case Transfer::TransferType_EuTransfer:
          rv=euTransferFromTemplate((*it).cast<Transaction>(),
                                    getParentWidget());
          break;
        case Transfer::TransferType_Unknown:
          abort();
        }

        if (rv==GWEN_ERROR_USER_ABORTED) {
          DBG_ERROR(0, "User aborted");
          if (i)
            _flagStaff->transfersUpdated();
          return rv;
        }
        i++;
      } // for
      if (i)
        _flagStaff->transfersUpdated();
      else {
        DBG_ERROR(0, "No transfers selected...");
      }
    }
  }
  else {
    DBG_INFO(0, "Importing transactions...");
    if (!imp.importContext()) {
      DBG_INFO(0, "Error importing transactions");
      return false;
    }

    _flagStaff->accountsUpdated();
    _flagStaff->transfersUpdated();
    _flagStaff->datedTransfersUpdated();
    _flagStaff->standingOrdersUpdated();
    _flagStaff->payeesUpdated();
  }

  return true;
}



void KBanking::infoTransactionsChanged(Account *a){
  _flagStaff->accountsUpdated();
  appInfoTransactionsChanged(a);
}



void KBanking::rebuildExportMenu(){
  _flagStaff->rebuildExportMenu();
}



int KBanking::init(){
  GWEN_DB_NODE *dbConfig;
  int rv;
  uint32_t lastVersion;

  rv=App::init();
  if (rv) {
    DBG_ERROR(0, "here (%d)", rv);
    return rv;
  }

  lastVersion=getLastVersion();
  if ((lastVersion!=APP_LAST_VERSION_NONE) &&
      (lastVersion<
       ((0<<24)+
        (9<<16)+
        (18<<8)+
        5))) {
    DBG_NOTICE(0, "Updating from old version, updating transactions");
    CheckDuplicates w(this);
    w.show();
    w.adjustDates();
  }

  if ((lastVersion!=APP_LAST_VERSION_NONE) &&
      (lastVersion<
       ((0<<24)+
        (9<<16)+
        (28<<8)+
        1))) {
    DBG_NOTICE(0, "Updating from old version, updating transactions");
    CheckDuplicates w(this);
    w.show();
    w.gatherInfoFromPurpose();
  }

  rv=loadAppSubConfig("kbanking/app",
		      &dbConfig,
		      0);
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
  }
  else {
    _lastReportId=GWEN_DB_GetIntValue(dbConfig, "lastReportId", 0, 0);
    _useColoursForCategoryText=GWEN_DB_GetIntValue(dbConfig,
						   "useColoursForCategoryText",
						   0, 1)!=0;
  }
  _sampleReportModules();

  rv=lockSharedConfig("certs", 0);
  if (rv<0) {
    DBG_ERROR(0, "ERROR: Could not lock certs db (%d)", rv);
  }
  else {
    GWEN_DB_NODE *dbCerts=NULL;

    rv=loadSharedConfig("certs", &dbCerts, 0);
    if (rv<0) {
      DBG_ERROR(0, "ERROR: Could not load certs (%d)", rv);
    }
    else {
      getGui()->setDbCerts(dbCerts);
    }
    rv=unlockSharedConfig("certs", 0);
    if (rv<0) {
      DBG_ERROR(0, "ERROR: Could not unlock certs db (%d)", rv);
    }
  }

  return 0;
}



int KBanking::fini(){
  GWEN_DB_NODE *dbConfig;
  GWEN_DB_NODE *dbCerts;
  int rv;

  dbConfig=GWEN_DB_Group_new("config");
  assert(dbConfig);

  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "lastReportId",
                      _lastReportId);

  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_OVERWRITE_VARS,
		      "useColoursForCategoryText",
		      _useColoursForCategoryText?1:0);

  rv=saveAppSubConfig("kbanking/app",
		      dbConfig,
		      0);
  GWEN_DB_Group_free(dbConfig);
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }

  /* store certificates */
  dbCerts=getGui()->getDbCerts();
  if (dbCerts) {
    rv=lockSharedConfig("certs", 0);
    if (rv<0) {
      DBG_ERROR(0, "ERROR: Could not lock certs db (%d)", rv);
    }
    else {
      rv=saveSharedConfig("certs", dbCerts, 0);
      if (rv<0) {
	DBG_ERROR(0, "ERROR: Could not save certs (%d)", rv);
      }
      rv=unlockSharedConfig("certs", 0);
      if (rv<0) {
	fprintf(stderr, "ERROR: Could not unlock certs db (%d)\n", rv);
      }
    }
  }

  return App::fini();
}



bool KBanking::useColoursForCategoryText() const {
  return _useColoursForCategoryText;
}



void KBanking::setUseColoursForCategoryText(bool b) {
  _useColoursForCategoryText=b;
}



void KBanking::setMainWindow(MainWindow *w){
  _mainWindow=w;
}



MainWindow *KBanking::getMainWindow(){
  return _mainWindow;
}



Report *KBanking::findReport(const char *name){
  std::list<Report*>::iterator it;

  for (it=_reports.begin(); it!=_reports.end(); it++) {
    std::string s;

    s=QStringToUtf8String((*it)->name());
    if (strcasecmp(s.c_str(), name)==0)
      return *it;
  } // for

  return 0;
}



std::list<Report*> KBanking::getReports(){
  return _reports;
}



GWEN_DB_NODE *KBanking::getReportProfiles(Report *r) {
  GWEN_DB_NODE *dbConfig=NULL;
  GWEN_BUFFER *nbuf;
  std::string name;
  std::string path;
  int rv;

  name=QStringToUtf8String(r->name());
  path="kbanking/reports";

  nbuf=GWEN_Buffer_new(0, 256, 0, 1);
  if (GWEN_Text_EscapeToBuffer(name.c_str(), nbuf)) {
    DBG_ERROR(0, "Could not escape report name");
    GWEN_Buffer_free(nbuf);
    return 0;
  }
  path+="/";
  path+=GWEN_Buffer_GetStart(nbuf);
  path+="/profiles";
  GWEN_Buffer_free(nbuf);

  rv=loadAppSubConfig(path.c_str(),
		      &dbConfig,
		      0);
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return NULL;
  }

  return dbConfig;
}



GWEN_DB_NODE *KBanking::findReportProfile(Report *r, const char *id){
  GWEN_DB_NODE *dbConfig;
  GWEN_DB_NODE *db;

  dbConfig=getReportProfiles(r);
  if (!dbConfig)
    return 0;
  db=GWEN_DB_FindFirstGroup(dbConfig, "profile");
  while(db) {
    const char *s;

    s=GWEN_DB_GetCharValue(db, "id", 0, 0);
    if (!s) {
      DBG_WARN(0, "Report with no id detected");
    }
    else {
      if (strcasecmp(s, id)==0) {
	GWEN_DB_UnlinkGroup(db);
	GWEN_DB_Group_free(dbConfig);
	return db;
      }
    }
    db=GWEN_DB_FindNextGroup(db, "profile");
  } // while
  GWEN_DB_Group_free(dbConfig);

  return NULL;
}



int KBanking::addReportProfile(Report *r, GWEN_DB_NODE *dbProfile){
  GWEN_DB_NODE *dbConfig=NULL;
  GWEN_BUFFER *nbuf;
  std::string name;
  std::string path;
  int rv;

  name=QStringToUtf8String(r->name());
  path="kbanking/reports";
  nbuf=GWEN_Buffer_new(0, 256, 0, 1);
  if (GWEN_Text_EscapeToBuffer(name.c_str(), nbuf)) {
    DBG_ERROR(0, "Could not escape report name");
    GWEN_Buffer_free(nbuf);
    return GWEN_ERROR_INVALID;
  }
  path+="/";
  path+=GWEN_Buffer_GetStart(nbuf);
  path+="/profiles";
  GWEN_Buffer_free(nbuf);

  rv=loadAppSubConfig(path.c_str(),
		      &dbConfig,
		      0);
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }
  else {
    char numbuf[32];
    GWEN_DB_NODE *dbNew=GWEN_DB_GetGroup(dbConfig,
					 GWEN_PATH_FLAGS_CREATE_GROUP,
					 "profile");
    assert(dbNew);
    GWEN_DB_AddGroupChildren(dbNew, dbProfile);
    snprintf(numbuf, sizeof(numbuf), "%d", ++_lastReportId);
    GWEN_DB_SetCharValue(dbNew, GWEN_DB_FLAGS_OVERWRITE_VARS,
			 "id", numbuf);

    rv=saveAppSubConfig(path.c_str(),
			dbConfig,
			0);
    GWEN_DB_Group_free(dbConfig);
    if (rv<0) {
      DBG_INFO(0, "here (%d)", rv);
      return rv;
    }
  }

  return 0;
}



void KBanking::removeReportProfile(Report *r,
                                   GWEN_DB_NODE *dbProfile){
  DBG_ERROR(0, "Not yet implemented");
}



void KBanking::_sampleReportModules(){
  Report *r;

  r=new SimpleReport(this);
  _reports.push_back(r);
  r=new SummaryReport(this);
  _reports.push_back(r);
  r=new CategoryReport(this);
  _reports.push_back(r);
  r=new GraphReport(this);
  _reports.push_back(r);
}



void KBanking::addTransactionWindow(TransactionWindow *w) {
  _tWindows.push_back(w);
}



void KBanking::delTransactionWindow(TransactionWindow *w) {
  std::list<RefPointer<TransactionWindow> >::iterator it;

  for (it=_tWindows.begin();
       it!=_tWindows.end();
       it++) {
    if ((*it).ptr()==w) {
      // found
      _tWindows.erase(it);
      return;
    }
  }
}



void KBanking::delTransactionWindows() {
  std::list<RefPointer<TransactionWindow> >::iterator it;

  while(!_tWindows.empty()) {
    RefPointer<TransactionWindow> wp=_tWindows.front();
    wp.ref().close();
  }
}



int KBanking::_newTransferJob(Transfer::TransferType tt,
                              RefPointer<Transfer> tmpl,
			      AB_JOB **retjob,
			      QWidget *parent){
  RefPointer<Transfer> t;
  bool b;
  int rv;
  AB_JOB *job=0;

  if (tmpl.isValid()) {
    const AB_VALUE *val;

    t=new Transfer(tmpl.ref());
    if (t.ref().getRemoteName().empty() ||
	t.ref().getRemoteBankCode().empty() ||
	t.ref().getRemoteAccountNumber().empty()) {
      std::string pname;

      /* we need to fill some fields from the payee */
      pname=t.ref().getPayee();
      if (!pname.empty()) {
	Payee *payee;

        payee=findPayeeById(pname.c_str());
	if (payee) {
	  std::string s;

	  s=payee->name();
          if (!s.empty())
	    t.ref().addRemoteName(payee->name());
	  if (t.ref().getRemoteBankCode().empty())
	    t.ref().setRemoteBankCode(payee->bankCode());
	  if (t.ref().getRemoteAccountNumber().empty())
	    t.ref().setRemoteAccountNumber(payee->accountNumber());
	}
      }
    }
    val=t.ref().getValue();
    if (val) {
      if (AB_Value_IsNegative(val)) {
	AB_VALUE *nv;

	nv=AB_Value_dup(val);
	AB_Value_Negate(nv);
	t.ref().setValue(nv);
        AB_Value_free(nv);
      }
    }
  }
  else {
    t=new Transfer(tt);
    if (tt==Transfer::TransferType_DebitNote)
      t.ref().setTextKey(5);
    else
      t.ref().setTextKey(51);
  }
  t.ref().setTransactionId(getNextUniqueId());

  while(1) {
#if 0
    GWEN_DB_NODE *dbT;
#endif
    int bad=false;
    AB_BANKINFO_CHECKRESULT res;
    int i;
    AB_ACCOUNT *ba;
    Account *acc;

    switch(tt) {
    case Transfer::TransferType_Simple:
      b=EditTransaction::editTransfer(this, t,
                                      QWidget::tr("New Transfer"),
                                      parent, true);
      break;
    case Transfer::TransferType_DebitNote:
      b=EditTransaction::editDebitNote(this, t,
                                       QWidget::tr("New Debit Note"),
                                       parent, true);
      break;
    case Transfer::TransferType_EuTransfer:
      b=EditEuTransaction::editEuTransfer(this, t,
                                          QWidget::tr("New Euro Transfer"),
                                          parent, true);
      break;
    default:
      b=false;
    }

    if (!b) {
      DBG_INFO(0, "User aborted");
      AB_Job_free(job);
      return GWEN_ERROR_USER_ABORTED;
    }

    // dump transaction
    DBG_NOTICE(0, "Accepted");
#if 0
    dbT=GWEN_DB_Group_new("Transaction");
    if (!t.ref().toDb(dbT)) {
      DBG_ERROR(0, "Could not write DB");
    }
    else {
#ifndef WIN32
	  // In MSVC this function crashes
      GWEN_DB_Dump(dbT, stderr, 2);
#endif
    }
    GWEN_DB_Group_free(dbT);
#endif

    // get account for this new transaction
    acc=findAccount(t.ref().getLocalBankCode().c_str(),
                          t.ref().getLocalAccountNumber().c_str());
    assert(acc);
    ba=getAccount(acc->getBankingId());
    if (!ba) {
      DBG_ERROR(0, "Account not available");
      QMessageBox::critical(parent,
                            QWidget::tr("Account Not Available"),
                            QWidget::tr("The account you requested is not "
                               "available\n"
                               "with any backend."),
                            QWidget::tr("Dismiss"), QString::null);
      return GWEN_ERROR_GENERIC;
    }

    // create job
    switch(tt) {
    case Transfer::TransferType_Simple:
      job=AB_JobSingleTransfer_new(ba);
      break;
    case Transfer::TransferType_DebitNote:
      job=AB_JobSingleDebitNote_new(ba);
      break;
    case Transfer::TransferType_EuTransfer:
      job=AB_JobEuTransfer_new(ba);
      break;
    default:
      return GWEN_ERROR_INVALID;
    }

    rv=AB_Job_CheckAvailability(job, 0);
    if (rv) {
      DBG_NOTICE(0, "Job is not available (%d)", rv);
      AB_Job_free(job);
      QMessageBox::critical(parent,
                            QWidget::tr("Job Not Available"),
                            QWidget::tr("The job you requested is not "
                               "available with\n"
                               "the backend which handles this account.\n"),
                            QWidget::tr("Dismiss"), QString::null);
      AB_Job_free(job);
      return GWEN_ERROR_NOT_AVAILABLE;
    }

    res=AB_Banking_CheckAccount(getCInterface(),
                                t.ref().getRemoteCountry().c_str(),
                                0,
                                t.ref().getRemoteBankCode().c_str(),
                                t.ref().getRemoteAccountNumber().c_str());
    switch(res) {
    case AB_BankInfoCheckResult_NotOk:
      DBG_INFO(0, "Account check result: %d", res);
      i=QMessageBox::warning(parent,
                             QWidget::tr("Warning"),
                             QWidget::tr("<qt>"
                                "<p>"
                                "The given bank code and account number "
                                "combination is invalid."
                                "</p>"
                                "Do you want to review the transfer?"
                                "<qt>"
                               ),
                             QMessageBox::Yes,
                             QMessageBox::No,
                             QMessageBox::Abort);
      if (i==0 || i == QMessageBox::Yes)
        bad=true;
      else if (i != 1 && i != QMessageBox::No) {
        AB_Job_free(job);
        return GWEN_ERROR_USER_ABORTED;
      }
      break;

    case AB_BankInfoCheckResult_UnknownBank:
      DBG_INFO(0, "Account check result: %d", res);
      i=QMessageBox::warning(parent,
                             QWidget::tr("Warning"),
                             QWidget::tr("<qt>"
                                "<p>"
                                "The given bank code is invalid."
                                "</p>"
                                "Do you want to review the transfer?"
                                "<qt>"
                               ),
                             QMessageBox::Yes,
                             QMessageBox::No,
                             QMessageBox::Abort);
      if (i == 0 || i == QMessageBox::Yes)
        bad=true;
      else if (i != 1 && i != QMessageBox::No) {
        AB_Job_free(job);
        return GWEN_ERROR_USER_ABORTED;
      }
      break;

    case AB_BankInfoCheckResult_UnknownResult:
      DBG_INFO(0, "Nothing known about account");
      break;
    case AB_BankInfoCheckResult_Ok:
      DBG_INFO(0, "Bank code/account number combination is ok");
      break;
    default:
      break;
    }

    if (!bad) {
      AB_TRANSACTION *abt;

      abt=t.ref().toBankingTransaction();
      assert(abt);

      switch(tt) {
      case Transfer::TransferType_Simple:
        rv=AB_JobSingleTransfer_SetTransaction(job, abt);
        break;
      case Transfer::TransferType_DebitNote:
        rv=AB_JobSingleDebitNote_SetTransaction(job, abt);
        break;
      case Transfer::TransferType_EuTransfer:
        rv=AB_JobEuTransfer_SetTransaction(job, abt);
        break;
      default:
        AB_Transaction_free(abt);
        AB_Job_free(job);
        return GWEN_ERROR_INVALID;
      }

      if (rv) {
        DBG_NOTICE(0, "Could not set transfer (%d)", rv);
        int res = QMessageBox::critical(parent,
                                  QWidget::tr("Bad Transfer Data"),
                                  QWidget::tr("<qt>"
                                     "<p>"
                                     "There seems to be some errors "
                                     "in the transfer data."
                                     "</p>"
                                     "<p>"
                                     "Do you want to review the transfer?"
                                     "</p>"
                                     "<qt>"
                                    ),
                                  QMessageBox::Yes,QMessageBox::Abort);
	if (res !=0 && res != QMessageBox::Yes){
          AB_Transaction_free(abt);
          AB_Job_free(job);
          return GWEN_ERROR_INVALID;
        }
        bad=true;
      }
      AB_Transaction_free(abt);
    } // if ok

    if (!bad) {
      break;
    }
    AB_Job_free(job);
    job=0;
  } // while

  *retjob=job;
  return 0;
}



int KBanking::_newTransfer(RefPointer<Transfer> tmpl,
                               QWidget *parent) {
  AB_JOB *job;
  int rv;

  rv=_newTransferJob(Transfer::TransferType_Simple, tmpl, &job, parent);
  if (rv) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }
  else {
    int rv;
    const AB_TRANSACTION *bat;

    assert(job);
    DBG_NOTICE(0, "Enqueuing job");
    rv=enqueueJob(job);
    if (rv) {
      DBG_NOTICE(0, "Error %d", rv);
      AB_Job_free(job);
      return rv;
    }
    bat=AB_JobSingleTransfer_GetTransaction(job);
    if (bat) {
      RefPointer<Transfer> t;
      GWEN_TIME *ti;

      t=new Transfer(bat, Transfer::TransferType_Simple);
      ti=GWEN_CurrentTime();
      if (ti) {
        t.ref().setDate(ti);
        GWEN_Time_free(ti);
      }
      t.ref().setStatus(Transfer::StatusEnqueued);
      t.ref().setJobId(AB_Job_GetJobId(job));
      addTransfer(t);
      transfersUpdated();
    }
    AB_Job_free(job);

    QMessageBox::information(parent,
                             QWidget::tr("Job enqueued"),
                             QWidget::tr("<qt>"
                                "A transfer job has been enqueued.\n"
                                "Please go to the <i>outbox</i> and press\n"
                                "the <i>execute</i> button there to "
                                "actually send\n"
                                "the enqueued jobs."
                                "</qt>"),
                             QWidget::tr("Dismiss"), QString::null);
  }

  return 0;
}



int KBanking::_newDebitNote(RefPointer<Transfer> tmpl,
                                QWidget *parent) {
  AB_JOB *job;
  int rv;

  rv=_newTransferJob(Transfer::TransferType_DebitNote, tmpl,
                     &job,
                     parent);
  if (rv) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }
  else {
    int rv;
    const AB_TRANSACTION *bat;

    DBG_NOTICE(0, "Enqueuing job");
    rv=enqueueJob(job);
    if (rv) {
      DBG_NOTICE(0, "Error %d", rv);
      AB_Job_free(job);
      return rv;
    }
    bat=AB_JobSingleDebitNote_GetTransaction(job);
    if (bat) {
      RefPointer<Transfer> t;
      GWEN_TIME *ti;

      t=new Transfer(bat, Transfer::TransferType_DebitNote);
      ti=GWEN_CurrentTime();
      if (ti) {
        t.ref().setDate(ti);
        GWEN_Time_free(ti);
      }
      t.ref().setStatus(Transfer::StatusEnqueued);
      t.ref().setJobId(AB_Job_GetJobId(job));
      addTransfer(t);
      transfersUpdated();
    }
    AB_Job_free(job);

    QMessageBox::information(parent,
                             QWidget::tr("Job enqueued"),
                             QWidget::tr("<qt>"
                                "A debit note job has been enqueued.\n"
                                "Please go to the <i>outbox</i> and press\n"
                                "the <i>execute</i> button there to "
                                "actually send\n"
                                "the enqueued jobs."
                                "</qt>"),
                             QWidget::tr("Dismiss"), QString::null);
  }

  return 0;
}



int KBanking::_newEuTransfer(RefPointer<Transfer> tmpl,
                             QWidget *parent) {
  AB_JOB *job;
  int rv;

  rv=_newTransferJob(Transfer::TransferType_EuTransfer, tmpl,
                     &job,
                     parent);
  if (rv) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }
  else {
    int rv;
    const AB_TRANSACTION *bat;

    DBG_NOTICE(0, "Enqueuing job");
    rv=enqueueJob(job);
    if (rv) {
      DBG_NOTICE(0, "Error %d", rv);
      AB_Job_free(job);
      return rv;
    }
    bat=AB_JobEuTransfer_GetTransaction(job);
    if (bat) {
      RefPointer<Transfer> t;
      GWEN_TIME *ti;

      t=new Transfer(bat, Transfer::TransferType_EuTransfer);
      ti=GWEN_CurrentTime();
      if (ti) {
        t.ref().setDate(ti);
        GWEN_Time_free(ti);
      }
      t.ref().setStatus(Transfer::StatusEnqueued);
      t.ref().setJobId(AB_Job_GetJobId(job));
      addTransfer(t);
      transfersUpdated();
    }
    AB_Job_free(job);

    QMessageBox::information(parent,
                             QWidget::tr("Job enqueued"),
                             QWidget::tr("<qt>"
                                "An EU-transfer job has been enqueued.\n"
                                "Please go to the <i>outbox</i> and press\n"
                                "the <i>execute</i> button there to "
                                "actually send\n"
                                "the enqueued jobs."
                                "</qt>"),
                             QWidget::tr("Dismiss"), QString::null);
  }

  return 0;
}



int KBanking::_newStandingOrderJob(RefPointer<Transaction> tmpl,
				   AB_JOB **retjob,
				   QWidget *parent){
  RefPointer<StandingOrder> t;
  bool b;
  int rv;
  AB_JOB *job=0;

  if (tmpl.isValid()) {
    const AB_VALUE *val;

    t=new StandingOrder(tmpl.ref());
    if (t.ref().getRemoteName().empty() ||
	t.ref().getRemoteBankCode().empty() ||
	t.ref().getRemoteAccountNumber().empty()) {
      std::string pname;

      /* we need to fill some fields from the payee */
      pname=t.ref().getPayee();
      if (!pname.empty()) {
	Payee *payee;

        payee=findPayeeById(pname.c_str());
	if (payee) {
	  std::string s;

	  s=payee->name();
          if (!s.empty())
	    t.ref().addRemoteName(payee->name());
	  if (t.ref().getRemoteBankCode().empty())
	    t.ref().setRemoteBankCode(payee->bankCode());
	  if (t.ref().getRemoteAccountNumber().empty())
	    t.ref().setRemoteAccountNumber(payee->accountNumber());
	}
      }
    }
    val=t.ref().getValue();
    if (val) {
      if (AB_Value_IsNegative(val)) {
	AB_VALUE *nv;

	nv=AB_Value_dup(val);
	AB_Value_Negate(nv);
	t.ref().setValue(nv);
        AB_Value_free(nv);
      }
    }
  }
  else {
    t=new StandingOrder();
    t.ref().setTextKey(51);
  }
  t.ref().setTransactionId(getNextUniqueId());

  while(1) {
    GWEN_DB_NODE *dbT;
    int bad=false;
    AB_BANKINFO_CHECKRESULT res;
    int i;
    AB_ACCOUNT *ba;
    Account *acc;

    b=EditStandingOrder::createStandingOrder(this, t,
					     QWidget::tr("New Standing Order"),
					     parent, true);
    if (!b) {
      DBG_INFO(0, "User aborted");
      AB_Job_free(job);
      return GWEN_ERROR_USER_ABORTED;
    }

    // dump transaction
    DBG_NOTICE(0, "Accepted");
    dbT=GWEN_DB_Group_new("Transaction");
    if (!t.ref().toDb(dbT)) {
      DBG_ERROR(0, "Could not write DB");
    }
    else {
#ifndef WIN32
	  // In MSVC this function crashes
      GWEN_DB_Dump(dbT, stderr, 2);
#endif
    }
    GWEN_DB_Group_free(dbT);


    // get account for this new transaction
    acc=findAccount(t.ref().getLocalBankCode().c_str(),
                          t.ref().getLocalAccountNumber().c_str());
    assert(acc);
    ba=getAccount(acc->getBankingId());
    if (!ba) {
      DBG_ERROR(0, "Account not available");
      QMessageBox::critical(parent,
                            QWidget::tr("Account Not Available"),
                            QWidget::tr("The account you requested is not "
                               "available\n"
                               "with any backend."),
                            QWidget::tr("Dismiss"), QString::null);
      return GWEN_ERROR_GENERIC;
    }

    // create job
    job=AB_JobCreateStandingOrder_new(ba);

    rv=AB_Job_CheckAvailability(job, 0);
    if (rv) {
      DBG_NOTICE(0, "Job is not available (%d)", rv);
      AB_Job_free(job);
      QMessageBox::critical(parent,
                            QWidget::tr("Job Not Available"),
                            QWidget::tr("The job you requested is not "
                               "available with\n"
                               "the backend which handles this account.\n"),
                            QWidget::tr("Dismiss"), QString::null);
      AB_Job_free(job);
      return GWEN_ERROR_NOT_AVAILABLE;
    }

    res=AB_Banking_CheckAccount(getCInterface(),
                                t.ref().getRemoteCountry().c_str(),
                                0,
                                t.ref().getRemoteBankCode().c_str(),
                                t.ref().getRemoteAccountNumber().c_str());
    switch(res) {
    case AB_BankInfoCheckResult_NotOk:
      DBG_INFO(0, "Account check result: %d", res);
      i=QMessageBox::warning(parent,
                             QWidget::tr("Warning"),
                             QWidget::tr("<qt>"
                                "<p>"
                                "The given bank code and account number "
                                "combination is invalid."
                                "</p>"
                                "Do you want to review the transfer?"
                                "<qt>"
                               ),
                             QMessageBox::Yes,
                             QMessageBox::No,
                             QMessageBox::Abort);
      if (i==0 || i == QMessageBox::Yes)
        bad=true;
      else if (i != 1 && i != QMessageBox::No) {
        AB_Job_free(job);
        return GWEN_ERROR_USER_ABORTED;
      }
      break;

    case AB_BankInfoCheckResult_UnknownBank:
      DBG_INFO(0, "Account check result: %d", res);
      i=QMessageBox::warning(parent,
                             QWidget::tr("Warning"),
                             QWidget::tr("<qt>"
                                "<p>"
                                "The given bank code is invalid."
                                "</p>"
                                "Do you want to review the transfer?"
                                "<qt>"
                               ),
                             QMessageBox::Yes,
                             QMessageBox::No,
                             QMessageBox::Abort);
      if (i == 0 || i == QMessageBox::Yes)
        bad=true;
      else if (i != 1 && i != QMessageBox::No) {
        AB_Job_free(job);
        return GWEN_ERROR_USER_ABORTED;
      }
      break;

    case AB_BankInfoCheckResult_UnknownResult:
      DBG_INFO(0, "Nothing known about account");
      break;
    case AB_BankInfoCheckResult_Ok:
      DBG_INFO(0, "Bank code/account number combination is ok");
      break;
    default:
      break;
    }

    if (!bad) {
      AB_TRANSACTION *abt;

      abt=t.ref().toBankingTransaction();
      assert(abt);

      rv=AB_JobCreateStandingOrder_SetTransaction(job, abt);
      if (rv) {
	DBG_NOTICE(0, "Could not set standing order (%d)", rv);
	int res = QMessageBox::critical(parent,
					QWidget::tr("Bad Transfer Data"),
					QWidget::tr("<qt>"
                                     "<p>"
                                     "There seems to be some errors "
                                     "in the transfer data."
                                     "</p>"
                                     "<p>"
                                     "Do you want to review the transfer?"
                                     "</p>"
                                     "<qt>"
                                    ),
                                  QMessageBox::Yes,QMessageBox::Abort);
	if (res !=0 && res != QMessageBox::Yes){
	  AB_Transaction_free(abt);
	  AB_Job_free(job);
	  return GWEN_ERROR_INVALID;
	}
	bad=true;
      }
      AB_Transaction_free(abt);
    } // if ok

    if (!bad) {
      break;
    }
    AB_Job_free(job);
    job=0;
  } // while

  *retjob=job;
  return 0;
}



int KBanking::_newDatedTransferJob(RefPointer<Transaction> tmpl,
				   AB_JOB **retjob,
				   QWidget *parent){
  RefPointer<Transfer> t;
  bool b;
  int rv;
  AB_JOB *job=0;

  if (tmpl.isValid()) {
    const AB_VALUE *val;

    t=new Transfer(tmpl.ref());
    if (t.ref().getRemoteName().empty() ||
	t.ref().getRemoteBankCode().empty() ||
	t.ref().getRemoteAccountNumber().empty()) {
      std::string pname;

      /* we need to fill some fields from the payee */
      pname=t.ref().getPayee();
      if (!pname.empty()) {
	Payee *payee;

        payee=findPayeeById(pname.c_str());
	if (payee) {
	  std::string s;

	  s=payee->name();
          if (!s.empty())
	    t.ref().addRemoteName(payee->name());
	  if (t.ref().getRemoteBankCode().empty())
	    t.ref().setRemoteBankCode(payee->bankCode());
	  if (t.ref().getRemoteAccountNumber().empty())
	    t.ref().setRemoteAccountNumber(payee->accountNumber());
	}
      }
    }
    val=t.ref().getValue();
    if (val) {
      if (AB_Value_IsNegative(val)) {
	AB_VALUE *nv;

	nv=AB_Value_dup(val);
	AB_Value_Negate(nv);
	t.ref().setValue(nv);
        AB_Value_free(nv);
      }
    }
  }
  else {
    t=new Transfer();
    t.ref().setTextKey(52);
  }

  while(1) {
    GWEN_DB_NODE *dbT;
    int bad=false;
    AB_BANKINFO_CHECKRESULT res;
    int i;
    AB_ACCOUNT *ba;
    Account *acc;

    b=EditTransaction::createDatedTransfer(this, t,
					   tr("New Dated Transfer"),
					   parent, true);
    if (!b) {
      DBG_INFO(0, "User aborted");
      AB_Job_free(job);
      return -1;
    }

    // dump transaction
    DBG_NOTICE(0, "Accepted");
    dbT=GWEN_DB_Group_new("DatedTransfer");
    if (!t.ref().toDb(dbT)) {
      DBG_ERROR(0, "Could not write DB");
    }
    else {
#ifndef WIN32
	  // In MSVC parent function crashes
      GWEN_DB_Dump(dbT, stderr, 2);
#endif
    }
    GWEN_DB_Group_free(dbT);


    // get account for parent new transaction
    acc=findAccount(t.ref().getLocalBankCode().c_str(),
		    t.ref().getLocalAccountNumber().c_str());
    assert(acc);
    ba=getAccount(acc->getBankingId());
    if (!ba) {
      DBG_ERROR(0, "Account not available");
      QMessageBox::critical(parent,
			    tr("Account Not Available"),
			    tr("The account you requested is not "
                               "available\n"
                               "with any backend."),
                            tr("Dismiss"), QString::null);
      return -1;
    }

    // create job
    job=AB_JobCreateDatedTransfer_new(ba);
    rv=AB_Job_CheckAvailability(job, 0);
    if (rv) {
      DBG_NOTICE(0, "Job is not available (%d)", rv);
      QMessageBox::critical(parent,
                            tr("Job Not Available"),
                            tr("The job you requested is not "
                               "available with\n"
                               "the backend which handles parent account.\n"),
                            tr("Dismiss"), QString::null);
      AB_Job_free(job);
      return -1;
    }

    res=AB_Banking_CheckAccount(getCInterface(),
                                t.ref().getRemoteCountry().c_str(),
                                0,
                                t.ref().getRemoteBankCode().c_str(),
                                t.ref().getRemoteAccountNumber().c_str());
    switch(res) {
    case AB_BankInfoCheckResult_NotOk:
      DBG_INFO(0, "Account check result: %d", res);
      i=QMessageBox::warning(parent,
                             tr("Warning"),
                             tr("<qt>"
                                "<p>"
                                "The given bank code and account number "
                                "combination is invalid."
                                "</p>"
                                "Do you want to review the order?"
                                "<qt>"
                               ),
                             QMessageBox::Yes,QMessageBox::No,QMessageBox::Abort);
      if (i==0 || i == QMessageBox::Yes)
        bad=true;
      else if (i!=1 && i != QMessageBox::No) {
        AB_Job_free(job);
        return -1;
      }
      break;

    case AB_BankInfoCheckResult_UnknownBank:
      DBG_INFO(0, "Account check result: %d", res);
      i=QMessageBox::warning(parent,
                             tr("Warning"),
                             tr("<qt>"
                                "<p>"
                                "The given bank code is invalid."
                                "</p>"
                                "Do you want to review the order?"
                                "<qt>"
                               ),
                             QMessageBox::Yes,QMessageBox::No,QMessageBox::Abort);
      if (i==0 || i == QMessageBox::Yes)
        bad=true;
      else if (i!=1 && i != QMessageBox::No) {
        AB_Job_free(job);
        return -1;
      }
      break;

    case AB_BankInfoCheckResult_UnknownResult:
      DBG_INFO(0, "Nothing known about account");
      break;
    case AB_BankInfoCheckResult_Ok:
      DBG_INFO(0, "Bank code/account number combination is ok");
      break;
    default:
      break;
    }

    if (!bad) {
      AB_TRANSACTION *abt;

      abt=t.ref().toBankingTransaction();
      assert(abt);

      rv=AB_JobCreateDatedTransfer_SetTransaction(job, abt);
      if (rv) {
        DBG_NOTICE(0, "Could not set transfer (%d)", rv);
        int res = QMessageBox::critical(parent,
                                  tr("Bad Transfer Data"),
                                  tr("<qt>"
                                     "<p>"
                                     "There seems to be some errors "
                                     "in the dated transfer data."
                                     "</p>"
                                     "<p>"
                                     "Do you want to review the order?"
                                     "</p>"
                                     "<qt>"
                                    ),
                                  QMessageBox::Yes,QMessageBox::Abort);
	if (res !=0 && res != QMessageBox::Yes){
          AB_Transaction_free(abt);
          AB_Job_free(job);
          return -1;
        }
        bad=true;
      }
      AB_Transaction_free(abt);
    } // if ok

    if (!bad) {
      break;
    }
    AB_Job_free(job);
    job=0;
  } // while

  *retjob=job;

  return 0;
}



int KBanking::transferFromTemplate(RefPointer<Transaction> tmpl,
                                   QWidget *parent) {
  int rv;

  if (tmpl.isValid()) {
    RefPointer<Transfer> t;

    t=new Transfer(tmpl.ref(), Transfer::TransferType_Simple);
    rv=_newTransfer(t, parent);
  }
  else
    rv=_newTransfer(0, parent);
  if (rv) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }

  return 0;
}



int KBanking::euTransferFromTemplate(RefPointer<Transaction> tmpl,
                                      QWidget *parent) {
  int rv;

  if (tmpl.isValid()) {
    RefPointer<Transfer> t;

    t=new Transfer(tmpl.ref(), Transfer::TransferType_EuTransfer);
    rv=_newEuTransfer(t, parent);
  }
  else
    rv=_newEuTransfer(0, parent);
  if (rv) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }

  return 0;
}



int KBanking::debitNoteFromTemplate(RefPointer<Transaction> tmpl,
                                    QWidget *parent) {
  int rv;

  if (tmpl.isValid()) {
    RefPointer<Transfer> t;

    t=new Transfer(tmpl.ref(), Transfer::TransferType_DebitNote);
    rv=_newDebitNote(t, parent);
  }
  else
    rv=_newDebitNote(0, parent);
  if (rv) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }

  return 0;
}



int KBanking::standingOrderFromTemplate(RefPointer<Transaction> tmpl,
					QWidget *parent) {
  AB_JOB *job;
  int rv;

  rv=_newStandingOrderJob(tmpl, &job, parent);
  if (rv) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }
  else {
    int rv;

    DBG_NOTICE(0, "Enqueuing job");
    rv=enqueueJob(job);
    if (rv) {
      DBG_NOTICE(0, "Error %d", rv);
      AB_Job_free(job);
      return rv;
    }
    AB_Job_free(job);

    QMessageBox::information(parent,
			     QWidget::tr("Job enqueued"),
			     QWidget::tr("<qt>"
			     "A Standing Order job has been enqueued.\n"
			     "Please go to the <i>outbox</i> and press\n"
			     "the <i>execute</i> button there to "
			     "actually send\n"
			     "the enqueued jobs."
			     "</qt>"),
                             QWidget::tr("Dismiss"), QString::null);
  }

  return 0;
}



int KBanking::datedTransferFromTemplate(RefPointer<Transaction> tmpl,
					QWidget *parent) {
  AB_JOB *job;
  int rv;

  rv=_newDatedTransferJob(tmpl, &job, parent);
  if (rv) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }
  else {
    int rv;
    const AB_TRANSACTION *bat;

    DBG_NOTICE(0, "Enqueuing job");
    rv=enqueueJob(job);
    if (rv) {
      DBG_NOTICE(0, "Error %d", rv);
      AB_Job_free(job);
      return rv;
    }
    bat=AB_JobCreateDatedTransfer_GetTransaction(job);
    if (bat) {
      RefPointer<Transfer> t;
      GWEN_TIME *ti;

      t=new Transfer(bat, Transfer::TransferType_Simple);
      ti=GWEN_CurrentTime();
      if (ti) {
        t.ref().setDate(ti);
        GWEN_Time_free(ti);
      }
      addDatedTransfer(t);
    }
    AB_Job_free(job);

    QMessageBox::information(parent,
                             QWidget::tr("Job enqueued"),
                             QWidget::tr("<qt>"
			     "A dated transfer job has been enqueued.\n"
			     "Please go to the <i>outbox</i> and press\n"
			     "the <i>execute</i> button there to "
			     "actually send\n"
			     "the enqueued jobs."
			     "</qt>"),
                             QWidget::tr("Dismiss"), QString::null);
  }

  return 0;
}



std::list<AB_JOB*> KBanking::getEnqueuedJobs(){
  AB_JOB_LIST2 *ll;
  std::list<AB_JOB*> rl;

  ll=_jobQueue;
  if (ll && AB_Job_List2_GetSize(ll)) {
    AB_JOB *j;
    AB_JOB_LIST2_ITERATOR *it;

    it=AB_Job_List2_First(ll);
    assert(it);
    j=AB_Job_List2Iterator_Data(it);
    assert(j);
    while(j) {
      rl.push_back(j);
      j=AB_Job_List2Iterator_Next(it);
    }
    AB_Job_List2Iterator_free(it);
  }
  return rl;
}



int KBanking::enqueueJob(AB_JOB *j) {
  assert(_jobQueue);
  assert(j);
  AB_Job_Attach(j);
  AB_Job_List2_PushBack(_jobQueue, j);
  _flagStaff->queueUpdated();
  return 0;
}



int KBanking::dequeueJob(AB_JOB *j) {
  assert(_jobQueue);
  AB_Job_List2_Remove(_jobQueue, j);
  AB_Job_free(j);
  _flagStaff->queueUpdated();
  return 0;
}




