/***************************************************************************
 $RCSfile$
                             -------------------
    cvs         : $Id$
    begin       : Mon Mar 01 2004
    copyright   : (C) 2004 by Martin Preuss
    email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "transactionwindow.h"
#include "kbanking.h"

#include <aqbanking/jobgetbalance.h>
#ifdef WIN32
# define strcasecmp stricmp
#endif

#include <qevent.h>
#include <qpushbutton.h>
#include <qgroupbox.h>
#include <qmessagebox.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qstatusbar.h>
#include <qlayout.h>

#include <gwenhywfar/debug.h>




TransactionWindow::TransactionWindow(Account *a,
                                     QWidget* parent,
                                     const char* name,
                                     WFlags fl)
:QWidget(parent, name, fl), _app(0), _account(a)
,_statusBar(0)
,_balance(0)
,_transactionView(0)
,_tname(0){
  QString bankName;
  QString accountName;
  QString balance;

  _app=dynamic_cast<KBanking*>(a->getApi());
  assert(_app);

  _app->addTransactionWindow(this);

  QBoxLayout *vBoxLayout = new QVBoxLayout(this, 5);
  vBoxLayout->setAlignment(Qt::AlignTop);
  _transactionView=new TransactionView(a, true, this, "TransactionView");
  _transactionView->setSizePolicy(QSizePolicy::Expanding,
                                  QSizePolicy::Expanding);
  vBoxLayout->addWidget(_transactionView);
  _statusBar=new QStatusBar(this);
  _statusBar->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
  vBoxLayout->addWidget(_statusBar);

  bankName=a->getBankName().c_str();
  if (bankName.isEmpty())
    bankName=a->getBankCode().c_str();
  _statusBar->addWidget(new QLabel(tr("Account"), this, 0), 0, true);
  _statusBar->addWidget(new QLabel(bankName, 0, "BankName"), 0, true);
  accountName=a->getAccountName().c_str();
  if (accountName.isEmpty())
    accountName=a->getAccountNumber().c_str();
  _statusBar->addWidget(new QLabel(accountName, this, "AccountName"), 0,
                        true);

  _balance=new QLabel(QString("----------"), this, "Balance");
  _statusBar->addWidget(_balance, 0, true);
}



TransactionWindow::TransactionWindow(KBanking *app,
                                     const std::list<RefPointer<Transaction> > &l,
                                     const char *tname,
                                     QWidget* parent,
                                     const char* name,
                                     WFlags fl)
:QWidget(parent, name, fl), _app(0), _account(0)
,_statusBar(0)
,_balance(0)
,_transactionView(0)
,_tname(tname){
  QString bankName;
  QString accountName;
  QString balance;

  _app=app;
  _app->addTransactionWindow(this);

  QBoxLayout *vBoxLayout = new QVBoxLayout(this, 5);
  vBoxLayout->setAlignment(Qt::AlignTop);
  _transactionView=new TransactionView(app,
                                       l,
                                       _tname,
                                       true,
                                       this,
                                       "TransactionView");
  _transactionView->setSizePolicy(QSizePolicy::Expanding,
                                  QSizePolicy::Expanding);
  vBoxLayout->addWidget(_transactionView);
  _statusBar=new QStatusBar(this);
  _statusBar->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
  vBoxLayout->addWidget(_statusBar);

}


TransactionWindow::~TransactionWindow(){
}



bool TransactionWindow::init(){
  const AB_ACCOUNT_STATUS *ast;
  GWEN_DB_NODE *dbConfig=NULL;
  std::string s;
  std::string path;
  int rv;

  path="gui/views/transactionwindow";
  if (_account) {
    s=_account->getBankCode();
    if (!s.empty()) {
      path+="/";
      path+=s;
    }
    s=_account->getAccountNumber();
    if (!s.empty()) {
      path+="/";
      path+=s;
    }
  }
  else if (_tname) {
    path+="/";
    path+=_tname;
  }
  else {
    path+="/generic";
  }

  rv=_app->loadAppSubConfig(path.c_str(),
			    &dbConfig,
			    0);
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
  }
  else {
    int w;
    int h;

    /* found settings */
    w=GWEN_DB_GetIntValue(dbConfig, "width", 0, 640);
    h=GWEN_DB_GetIntValue(dbConfig, "height", 0, 480);
    resize(w, h);

    GWEN_DB_Group_free(dbConfig);
  }

  if (!_transactionView->init())
    return false;

  if (_account) {
    ast=_account->getAccountStatus();
    if (ast) {
      QString s;
      const AB_BALANCE *bal;

      bal=AB_AccountStatus_GetBookedBalance(ast);
      if (bal) {
        const AB_VALUE *v;

        v=AB_Balance_GetValue(bal);
        if (v) {
          const char *c;

          s=QString::number(AB_Value_GetValueAsDouble(v), 'f', 2);
          c=AB_Value_GetCurrency(v);
          if (!c)
            c="EUR";
          s+=" ";
          s+=QString::fromUtf8(c);
        }
      }
      _balance->setText(s);
    }
    else {
      _balance->setText(QString::null);
    }
  }

  return true;
}



bool TransactionWindow::fini(){
  GWEN_DB_NODE *dbConfig;
  int rv;
  std::string s;
  std::string path;

  if (!_transactionView->fini())
    return false;

  path="gui/views/transactionwindow";
  if (_account) {
    s=_account->getBankCode();
    if (!s.empty()) {
      path+="/";
      path+=s;
    }
    s=_account->getAccountNumber();
    if (!s.empty()) {
      path+="/";
      path+=s;
    }
  }
  else if (_tname) {
    path+="/";
    path+=_tname;
  }
  else {
    path+="/generic";
  }

  dbConfig=GWEN_DB_Group_new("config");
  assert(dbConfig);

  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_DEFAULT,
                      "width", width());
  GWEN_DB_SetIntValue(dbConfig, GWEN_DB_FLAGS_DEFAULT,
                      "height", height());

  rv=_app->saveAppSubConfig(path.c_str(),
			    dbConfig,
			    0);
  GWEN_DB_Group_free(dbConfig);
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return false;
  }

  return true;
}



void TransactionWindow::slotUpdated() {
  if (_transactionView)
    _transactionView->slotUpdated();
}



void TransactionWindow::closeEvent(QCloseEvent *e){
  if (!fini()) {
    DBG_ERROR(0, "Could not deinit transaction window");
  }
  _app->delTransactionWindow(this);
  e->accept();
}






